package xsql.parse;

import java.util.*;
import java.lang.*;
import java.io.*;

%%

%implements Lexer
%function nextToken
%type java_cup.runtime.Symbol
%char
%line

%{
private void err(int pos, String s) {
  errorMsg.error(pos,s);
}

private void err(String s) {
  err(yychar,s);
}

private java_cup.runtime.Symbol tok(int kind, Object value) {
    return new java_cup.runtime.Symbol(kind, yychar, yychar+yylength(), value);
}

private xsql.error.ErrorMsg errorMsg;

Yylex(java.io.InputStream s, xsql.error.ErrorMsg e) {
  this(s);
  errorMsg=e;
}

%}

%eofval{
	{
	 return tok(sym.EOF, null);
        }
%eofval}       

NAME=[A-Za-z][A-Za-z0-9_]*

WHITE_SPACE_CHAR=[\f \t\b\012]

EOL = [\r\n]

INT_NUM = [0-9]+

QQSTRING = (\\\"|[^\n\"]|\\{WHITE_SPACE_CHAR}+\\)*

QSTRING = (\\\'|[^\n\']|\\{WHITE_SPACE_CHAR}+\\)*

CHARACTER = CHAR(ACTER)?

INTEGER = INT(EGER)?

COMPARISON = (=|<>|<|>|<=|>=)

%%

  ";" { return tok(sym.SEMI, null); }

  "," { return tok(sym.COMMA, null); }

  "." { return tok(sym.PERIOD, null); }

  "(" { return tok(sym.LPAREN, null); }

  ")" { return tok(sym.RPAREN, null); }

  "*" { return tok(sym.ASTERISK, null); }
  
  "AND" { return tok(sym.AND, null); }
  "and" { return tok(sym.AND, null); }

  "WHERE" { return tok(sym.WHERE, null); }
  "where" { return tok(sym.WHERE, null); }

  "INSERT" { return tok(sym.INSERT, null); }
  "insert" { return tok(sym.INSERT, null); }

  "UPDATE" { return tok(sym.UPDATE, null); }
  "update" { return tok(sym.UPDATE, null); }

  "DELETE" { return tok(sym.DELETE, null); }
  "delete" { return tok(sym.DELETE, null); }

  "SET" { return tok(sym.SET, null); }
  "set" { return tok(sym.SET, null); }

  "VALUES" { return tok(sym.VALUES, null); }
  "values" { return tok(sym.VALUES, null); }

  "INTO" { return tok(sym.INTO, null); }
  "into" { return tok(sym.INTO, null); }

  "SELECT" { return tok(sym.SELECT, null); }
  "select" { return tok(sym.SELECT, null); }

  "FROM" { return tok(sym.FROM, null); }
  "from" { return tok(sym.FROM, null); }

  "TABLE" { return tok(sym.TABLE, null); }
  "table" { return tok(sym.TABLE, null); }

  "CREATE" { return tok(sym.CREATE, null); }
  "create" { return tok(sym.CREATE, null); }

  "NUMERIC" { return tok(sym.NUMERIC, null); }
  "numeric" { return tok(sym.NUMERIC, null); }

  "DROP" { return tok(sym.DROP, null); }
  "drop" { return tok(sym.DROP, null); }

  "PRIMARY KEY" { return tok(sym.PRIMARY_KEY, null); }
  "primary key" { return tok(sym.PRIMARY_KEY, null); }

  {INTEGER} { return tok(sym.INTEGER, null); }

  {CHARACTER} { return tok(sym.CHARACTER, null); }

  {NAME} { return tok(sym.NAME, yytext()); }

  {COMPARISON} { return tok(sym.COMPARISON, yytext()); }

  {WHITE_SPACE_CHAR} { }

  {INT_NUM} { return tok(sym.INT_NUM, yytext()) ; }

  \"{QQSTRING}\" { return tok(sym.STRING, 
                            yytext().substring(1, yytext().length() - 1)) ; } 

  \'{QSTRING}\' { return tok(sym.STRING, 
                            yytext().substring(1, yytext().length() - 1)) ; } 

{EOL}	{ return tok(sym.EOF, null); }

.               { err("Lexer: Unknown input"); }

